#include <stdlib.h>
#include <stdio.h>

#include <gtk/gtk.h>
#include "gtkeditor.h"

#define BUF_MAX_SIZE 128	/* seems fair for one line */

static void
load_file (GtkEditor *editor, char *filename)
{
  char buf[BUF_MAX_SIZE];
  FILE *fp = fopen (filename, "r");

  if (!fp) return;

  g_print ("inserting %s\n", filename);

  /* load new */
  gtk_text_freeze (GTK_TEXT (editor));
  
  do {
    if (fgets (buf, BUF_MAX_SIZE, fp) == NULL) {
      break;
    } else {
      gtk_text_insert (GTK_TEXT (editor), NULL, NULL, NULL,
		       buf, strlen (buf));
    }
  } while (!feof (fp));

  gtk_text_thaw (GTK_TEXT (editor));

  /* clean up and hilite */
  fclose (fp);

  g_print ("hiliting\n");
  gtk_editor_hilite_buffer (editor);
  g_print ("done!");

  exit (2);			/* ready for profileing */
}


int
main (int argc, char *argv[]) 
{
  GtkWidget *window;
  GtkWidget *editor;
  GList *entries = NULL;

  GdkColor red;
  GdkColormap *cmap;
  GdkFont *commentfont;

  red.pixel = 0;
  red.red = 65535;
  red.green = 0;
  red.blue = 0;


  /* args? */
  if (argc != 3) {
    g_warning ("wrong user error!");
    return EXIT_FAILURE;
  }

  /* initialize */
  gtk_init (&argc, &argv);

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  editor = gtk_editor_new(NULL, NULL);

  cmap = gtk_widget_get_colormap (editor);
  commentfont =
    gdk_font_load ("-adobe-courier-medium-i-normal-*-12-*-*-*-*-*-*-*");
  gdk_color_alloc(cmap, &red);
  

  /* second argument selects syn-blocks */
  switch (atoi (argv[2])) {
  case 0:
    /* no syntax bloc */
    break;
  case 1:
    /* only C-comments */
    entries = gtk_editor_stentry_new ("C-comment",
				      "/\\*", "\\*/", FALSE,
				      commentfont, &red, NULL,
				      NULL);
    break;
  case 2:
    /* more comments */
    entries = gtk_editor_stentry_new ("C-comment",
				      "/\\*", "\\*/", FALSE,
				      commentfont, &red, NULL,
				      gtk_editor_stentry_new ("C++-comment",
							      "//", "\n", FALSE,
							      commentfont, &red, NULL,
							      NULL));
    break;
  case 3:
    /* comments *and* strings */
  entries = gtk_editor_stentry_new ("C-comment",
				    "/\\*", "\\*/", FALSE,
				    commentfont, &red, NULL,
				    gtk_editor_stentry_new ("C++-comment",
							    "//", "\n", FALSE,
							    commentfont, &red, NULL,
				    gtk_editor_stentry_new ("string",
							    "\"", "\"", FALSE,
							    NULL, &red, NULL,
				    gtk_editor_stentry_new ("chars",
							    "'", "'", FALSE,
							    NULL, &red, NULL,
							    NULL))));
  break;
  }

  gtk_editor_install_stable (GTK_EDITOR (editor), entries);

  gtk_container_add (GTK_CONTAINER (window), editor);
  gtk_signal_connect_after (GTK_OBJECT (editor), "realize",
			    GTK_SIGNAL_FUNC (load_file), (gpointer)argv[1]);

  gtk_widget_show_all (window);


  /* main loop */
  gtk_main ();

  return EXIT_SUCCESS;
}
